{% with attributes = attributes | default(new_attributes()) %}
    {% do attributes.set(html_type='button') %}
    {% do attributes.html_class.append('button') %}
    {% do attributes.html_class.append('button-secondary' if button_secondary | default(false) else 'button-primary') %}
    <button {{ attributes }}>{{ button_label }}</button>
{% endwith %}
