{% with attributes = attributes | default(new_attributes()) %}
    {% do attributes.html_class.append('button-close') %}
    {% set button_label = _('Close') %}
    {% include 'component/button-icon.html.j2' %}
{% endwith %}
